;--------------------------------------------------------------------------
;  crt0.s - Generic crt0.s for a Z80
;
;  Copyright (C) 2000, Michael Hope
;
;  This library is free software; you can redistribute it and/or modify it
;  under the terms of the GNU General Public License as published by the
;  Free Software Foundation; either version 2, or (at your option) any
;  later version.
;
;  This library is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with this library; see the file COPYING. If not, write to the
;  Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
;   MA 02110-1301, USA.
;
;  As a special exception, if you link this library with other files,
;  some of which are compiled with SDCC, to produce an executable,
;  this library does not by itself cause the resulting executable to
;  be covered by the GNU General Public License. This exception does
;  not however invalidate any other reasons why the executable file
;   might be covered by the GNU General Public License.
;--------------------------------------------------------------------------

	.module crt0
	.globl	_main

	.area	_HEADER (ABS)
	;; Reset vector
	.org 	0
	jp	init

	.org	0x08
	ei
	reti
	.org	0x10
	ei
	reti
	.org	0x18
	ei
	reti
	.org	0x20
	ei
	reti
	.org	0x28
	ei
	reti
	.org	0x30
	ei
	reti
	.org	0x38
	ei
	reti

	.org	0x100
init:
	;; Set stack pointer directly above top of memory.
	ld	sp,#0xE800

	call	___sdcc_external_startup

	;; Initialise global variables. Skip if __sdcc_external_startup returned
	;; non-zero value. Note: calling convention version 1 only.
	or	a, a
	call	Z, gsinit

	call	_main
	jp	_exit

;; IM2 Vektortabelle
;; 0x00 0 0 0 0 0 0 0 0 CTC Ch0
;; 0x02 0 0 0 0 0 0 1 0 CTC Ch1
;; 0x04 0 0 0 0 0 1 0 0 CTC Ch2
;; 0x06 0 0 0 0 0 1 1 0 CTC Ch3
;; 0x10 0 0 0 1 0 0 0 0 PIO A
;; 0x12 0 0 0 1 0 0 1 0 PIO B
;; 0x20 0 0 1 0 0 0 0 0 ChB Transmit Buffer Empty
;; 0x22 0 0 1 0 0 0 1 0 ChB External/Status Change
;; 0x24 0 0 1 0 0 1 0 0 ChB Receive Character Available
;; 0x26 0 0 1 0 0 1 1 0 ChB Special Receive Condition
;; 0x28 0 0 1 0 1 0 0 0 ChA Transmit Buffer Empty
;; 0x2A 0 0 1 0 1 0 1 0 ChA External/Status Change
;; 0x2C 0 0 1 0 1 1 0 0 ChA Receive Character Available
;; 0x2E 0 0 1 0 1 1 1 0 ChA Special Receive Condition

VECT_BASE =	0x200
	.org VECT_BASE
_int_vector_table::
	.rept	32
	.dw	0
	.endm

;	.org VECT_BASE
;_int_vector_table::
;	.org VECT_BASE+0	;CTC
;	.dw	0		; Ch 0
;	.dw	_ctc_ch1_isr	; Ch 1
;	.dw	0		; Ch 1
;	.dw	0		; Ch 2
;	.org VECT_BASE+0x20	;SIO
;	.dw	0		; Ch B
;	.dw	0		; Ch B
;	.dw	0		; Ch B
;	.dw	0		; Ch B
;	.dw	_sioa_tx_isr	; Ch A Transmit Buffer Empty
;	.dw	_sioa_ext_isr	; Ch A External/Status Change
;	.dw	_sioa_rx_isr	; Ch A Receive Character Available
;	.dw	_sioa_srx_isr	; Ch A Special Receive Condition


	;; Ordering of segments for the linker.
	.area	_HOME
	.area	_CODE
	.area	_INITIALIZER
	.area   _GSINIT
	.area   _GSFINAL

	.area	_DATA
	.area	_INITIALIZED
	.area	_BSEG
	.area   _BSS
	.area   _HEAP

	.area   _CODE
_exit::
	;; Exit - special code to the emulator
	ld	a,#0
	rst	0x08
1$:
	halt
	jr	1$

	.area   _GSINIT
gsinit::

	; Default-initialized global variables.
        ld      bc, #l__DATA
        ld      a, b
        or      a, c
        jr      Z, zeroed_data
        ld      hl, #s__DATA
        ld      (hl), #0x00
        dec     bc
        ld      a, b
        or      a, c
        jr      Z, zeroed_data
        ld      e, l
        ld      d, h
        inc     de
        ldir
zeroed_data:

	; Explicitly initialized global variables.
	ld	bc, #l__INITIALIZER
	ld	a, b
	or	a, c
	jr	Z, gsinit_next
	ld	de, #s__INITIALIZED
	ld	hl, #s__INITIALIZER
	ldir

gsinit_next:

	.area   _GSFINAL
	ret

